/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caimengci <caimengci_cm@deepin.com>
 *
 * Maintainer: caimengci <caimengci_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "reminderdialog.h"
#include "networkprotectionworker.h"

#include <DLabel>
#include <DFontSizeManager>
#include <DSpinner>
#include <DStyle>

#include <QDesktopWidget>
#include <QApplication>
#include <QWindow>
#include <QHBoxLayout>
#include <QTimer>
#include <QDebug>

DWIDGET_USE_NAMESPACE

static const int TIMEOUT_COUNT = 15;

ReminderDialog::ReminderDialog(DDialog *parent)
    : ReminderDialog("", "", parent)
{

}

ReminderDialog::ReminderDialog(const QString &id, const QString &appName, bool ask, bool disable, DDialog *parent)
    : DDialog(parent)
    , m_id(id)
    , m_appName(appName)
    , m_timer(new QTimer(this))
    , m_ask(ask)
    , m_disable(disable)
    , m_exit(false)
    , m_elapsedSeconds(TIMEOUT_COUNT)
{
    setCloseButtonVisible(false);
    this->setWindowFlag(Qt::WindowStaysOnTopHint);
    this->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    this->setModal(true);

    //触发关闭按钮需要退出当前进程
    connect(this, &DDialog::closed, [&]{
        qDebug() << "dialog exited";
        m_exit = true;
    });
    initUI();
    m_timer->setInterval(1000);
    m_timer->setSingleShot(false);
    connect(m_timer, &QTimer::timeout, [=]() {
        m_elapsedSeconds--;

        if (m_elapsedSeconds > 0) {
            if (m_ask) {
                setButtonText(2, tr("Disable (%1)").arg(m_elapsedSeconds));
            } else if (m_disable) {
                setButtonText(1, tr("OK (%1)").arg(m_elapsedSeconds));
            }
            return;
        }

        m_timer->stop();

        if (m_exit)
            return;

        if (m_ask) {
            //default disable
            qDebug() << " 15s timed out, default disable";
            Q_EMIT notifyTimedout(m_id, true);
        } else {
            Q_EMIT notifyTimedout(m_id, false);
        }
        if (!m_exit)
            close();
        m_exit = true;
    });
    m_timer->start();
}

ReminderDialog::~ReminderDialog()
{
    m_timer->stop();
    m_timer->deleteLater();
    qDebug() << "ungrab";
    bool bGrabMouseState = windowHandle()->setMouseGrabEnabled(false);
    bool bGrabKeyboardState = windowHandle()->setKeyboardGrabEnabled(false);
    qDebug() << "取消抓取鼠标焦点(true : 成功)，setMouseGrabEnabled :" << bGrabMouseState;
    qDebug() << "取消抓取键盘焦点(true : 成功)， d->setKeyboardGrabEnabled :" << bGrabKeyboardState;
}

void ReminderDialog::setId(const QString &name)
{
    m_id = name;
    initUI();
}

void ReminderDialog::grabMouseAndKeyboardFocus(bool bgrab)
{
    QDesktopWidget *desktop = qApp->desktop();
    move((desktop->width() - width()) / 2, (desktop->height() - height()) / 2);
    grab();

    if (!bgrab)
        return;

    bool bGrabMouseState = windowHandle()->setMouseGrabEnabled(true);
    bool bGrabKeyboardState = windowHandle()->setKeyboardGrabEnabled(true);
    qDebug() << "抓取鼠标焦点(true : 成功)，setMouseGrabEnabled :" << bGrabMouseState;
    qDebug() << "抓取键盘焦点(true : 成功)， d->setKeyboardGrabEnabled :" << bGrabKeyboardState;

    //获取鼠标焦点失败后，重抓，最多重连3次
    if (!bGrabMouseState) {
        QTimer::singleShot(100, this, [this]() {
            for (int i = 0; i < 3; i++) {
                bool state = windowHandle()->setMouseGrabEnabled(true);
                qDebug() << "抓取鼠标焦点(true : 成功)，setMouseGrabEnabled :" << state;
                if (state) {
                    break;
                }
            }
        });
    }

    //获取键盘焦点失败后，重抓，最多重抓3次
    if (!bGrabKeyboardState) {
        QTimer::singleShot(100, this, [this]() {
            for (int i = 0; i < 3; i++) {
                bool state = windowHandle()->setKeyboardGrabEnabled(true);
                qDebug() << "抓取键盘焦点(true : 成功)setKeyboardGrabEnabled :" << state;
                if (state) {
                    break;
                }
            }
        });
    }
}

void ReminderDialog::setAsk(bool ask)
{
    m_ask = ask;
}

void ReminderDialog::setDisable(bool disable)
{
    m_disable = disable;
}

void ReminderDialog::initUI()
{
    QStringList list = m_id.split("-");

    clearContents();
    clearButtons();
    DLabel *txt = nullptr;
    if (m_ask) {
        txt = new DLabel(tr("%1 wants to connect to the Internet").arg(m_appName));
        txt->setWordWrap(true);
        addContent(txt);
        addButton(tr("Allow Once"), true, DDialog::ButtonRecommend);
        addButton(tr("Allow Always"), true, DDialog::ButtonRecommend);
        addButton(tr("Disable (%1)").arg(m_elapsedSeconds), true, DDialog::ButtonRecommend);
    } else if (m_disable) {
        txt = new DLabel(tr("The firewall has blocked %1 from connecting to the Internet").arg(m_appName));
        txt->setWordWrap(true);
        addContent(txt);
        addButton(tr("Go to Settings"), true, DDialog::ButtonRecommend);
        addButton(tr("OK (%1)").arg(m_elapsedSeconds), true, DDialog::ButtonRecommend);
    }

    connect(this, &DDialog::buttonClicked, this, [ = ](int index, const QString & text) {
        Q_UNUSED(text)
        m_timer->stop();
        Q_EMIT notifyButtonClicked(m_id, index);
        m_exit = true;
        qDebug() << "exit---------";
    });
}

void ReminderDialog::mouseMoveEvent(QMouseEvent *e)
{
    Q_UNUSED(e)
}
