/*
 * Copyright (C) 2017 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     LiLinling <lilinling_cm@deepin.com>
 *
 * Maintainer: LiLinling <lilinling_cm@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "scanworker.h"
#include "virusmodel.h"

#include <QThread>
#include <QDateTime>
#include <QDebug>

const QString service = "com.deepin.defender.hmiscreen";
const QString path = "/com/deepin/defender/hmiscreen";

ScanWorker::ScanWorker(VirusModel *model, const QString &type, QWidget *parent)
    : m_parent(parent)
    , m_model(model)
    , m_isScanning(true)
    , m_selfPause(false)
    , m_totalFiles(0)
    , m_currentFileCnt(0)
{
    QString use_srv = service;
    if ("" != type)
        use_srv += QString(".%1").arg(type);

    QString use_path = path;
    if ("" != type)
        use_path += QString("/%1").arg(type);

    qDebug() << use_srv << use_path;
    m_defenderInter = new defenderInter(use_srv, use_path, QDBusConnection::sessionBus(), this);
    m_defenderInter->setSync(false);

    //注册自定义结构体
    registerDefenderFileInfoMetaType();
    registerDefenderDissectionMetaType();
    registerDefenderVersionInfoMetaType();
    registerDefenderUpdateStateMetaType();

    //关联获取实际数据变化后的信号
    connect(m_defenderInter, &defenderInter::dissectionInfoChanged, this, &ScanWorker::onDissectionInfoChanged);
    connect(m_defenderInter, &defenderInter::versionInfoChanged, this, &ScanWorker::onVersionInfoChanged);
    connect(m_defenderInter, &defenderInter::updateInfoChanged, this, &ScanWorker::onUpdateStateChanged);

    //关联状态变化信号,比如调用check(),刚调用发送"start",数据返回时发送"backing",返回数据结束发送"end"
    connect(m_defenderInter, &defenderInter::notifyDissectionStateChanged, this, &ScanWorker::onNotifyDissedtionStateChanged);
    connect(m_defenderInter, &defenderInter::notifyCheckStateChanged, this, &ScanWorker::onNotifyVersionInfoChanged);
    connect(m_defenderInter, &defenderInter::notifyUpdateStateChanged, this, &ScanWorker::onNotifyUpdateInfoChanged);

    //get virus lib version and date
    DefenderVersionInfo vif = m_defenderInter->versionInfo();
    qDebug() << "versionInfo version = " << vif.version << ", date = " << vif.time;
    m_model->setVirusVersion(vif.version);
    m_model->setVirusDate(QDateTime::fromMSecsSinceEpoch(vif.time).date().toString());
}

void ScanWorker::pause()
{
    m_isScanning = false;
}

void ScanWorker::resume()
{
    m_isScanning = true;
}

void ScanWorker::scan(const QString &path)
{
    QFileInfoList list = getAllFiles(path);

    for (QFileInfo fi : list) {
        qDebug() << fi.filePath();
        m_fileList << fi.filePath();
    }

    int m_totalFiles = list.count();

    if (m_currentFileCnt == 0)
        Q_EMIT totalItem(m_totalFiles);

    scanOne(m_currentFileCnt, m_totalFiles, m_fileList[m_currentFileCnt]);
}

void ScanWorker::scanOne(int current, int total, const QString &filePath)
{
    while (current < total) {
        if (!m_isScanning) {
            qDebug() << " is paused, waiting...";
            QThread::currentThread()->msleep(100);
            continue;
        }

        if (m_selfPause) {
            return;
        }

        m_selfPause = true;
        scanFile(filePath);
        //self pause to wait for scan result
    }
}

QFileInfoList ScanWorker::getAllFiles(const QString &path)
{
    QDir dir(path);
    QFileInfoList file_list = dir.entryInfoList(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
    QFileInfoList folder_list = dir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);

    for(int i = 0; i != folder_list.size(); i++)
    {
         QString name = folder_list.at(i).absoluteFilePath();
         QFileInfoList child_file_list = getAllFiles(name);
         file_list.append(child_file_list);
    }

    return file_list;
}

void ScanWorker::scanFile(const QString &path)
{
    m_defenderInter->dissection(path).waitForFinished();
}

void ScanWorker::scanProcess(const int processId)
{
    qDebug() << "begin scan process " << processId;
    m_defenderInter->dissection(processId).waitForFinished();
    qDebug() << "end scan process " << processId;
}

void ScanWorker::onVersionInfoChanged(DefenderVersionInfo info)
{
    qDebug() << Q_FUNC_INFO << info.time << info.version << info.description;
    QDateTime dt = QDateTime::fromMSecsSinceEpoch(info.time);
    m_model->setVirusDate(dt.date().toString());
    m_model->setVirusVersion(info.version);
}

void ScanWorker::onDissectionInfoChanged(DefenderDissection info)
{
//    qDebug() << Q_FUNC_INFO << info.taskId << info.progress << info.fileInfo.type << info.fileInfo.description << " thread id = " << QThread::currentThreadId();
    QString state = TypesNames.at(info.fileInfo.type);

    if (m_totalFiles == 0)
        m_totalFiles = m_fileList.size();
    ++m_currentFileCnt;
    int prog = m_currentFileCnt * 100 / m_totalFiles;
    qDebug() << "[debug test]progress value = " << prog;
    if (m_currentFileCnt == m_totalFiles - 1)
        prog = 100;

    int type = (info.fileInfo.type + m_currentFileCnt) % TypeMaxCnt;

    Q_EMIT progress(prog, type, info.fileInfo.description);
    m_selfPause = false;
    if (prog < 100) {
        scanOne(m_currentFileCnt, m_totalFiles, m_fileList[m_currentFileCnt]);
    } else {
        qDebug() << "all files finished. " << m_currentFileCnt << ", m_totalFiles = " << m_totalFiles;
        Q_EMIT over();
    }

}

void ScanWorker::onUpdateStateChanged(DefenderUpdateState info)
{
    qDebug() << Q_FUNC_INFO << info.progress << info.info.version << info.info.description << info.info.time;
}

void ScanWorker::onNotifyDissedtionStateChanged(int taskId, QString state)
{
    qDebug() << Q_FUNC_INFO << taskId << state;
}

void ScanWorker::onNotifyVersionInfoChanged(QString state)
{
    qDebug() << Q_FUNC_INFO << state;
}

void ScanWorker::onNotifyUpdateInfoChanged(QString state)
{
    qDebug() << Q_FUNC_INFO << state;
}
