/*
 * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
 *
 * Author:     caitao <caitao@deepin.com>
 *
 * Maintainer: caitao <caitao@deepin.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "examworker.h"

#include <QDebug>
#include <QThread>
#include <QDateTime>

//体检各阶段权重
const double SafetyLevelScoreCoe = 0.1;
const double AntivirusScoreCoe = 0.0;
const double NetProtectScoreCoe = 0.0;

ExamWorker::ExamWorker(QObject *parent)
    : QObject(parent)
    , m_mode(nullptr)
    , m_work(nullptr)
    , m_status(running)
{

}

ExamWorker::~ExamWorker()
{
    qDebug() << "examine worker quit";
}

void ExamWorker::pause()
{
    m_status = paused;
}

void ExamWorker::run()
{
    m_status = running;
}

void ExamWorker::resume()
{
    m_status = running;
}

void ExamWorker::quit()
{
    m_status = quitted;
    qDebug() << "quit";
}

void ExamWorker::judgeStatus()
{
    if (quitted == m_status) return;
    while (paused == m_status) {
        qDebug() << "examine worker is paused";
        thread()->msleep(100);
    }
}

void ExamWorker::waitWithStatus(int mSec)
{
    if (quitted == m_status) return;
    int time = 0;
    while (time < mSec) {
        judgeStatus();
        thread()->msleep(100);
        time += 100;
    }
}

//进行体检
//每个阶段调用sendExamineProcess，发送待修复项id和当前体检进度
//待修复项id对应于ExamDetailPage里的m_fixItemLst
void ExamWorker::examining()
{
    if (nullptr == m_work || nullptr == m_mode) {
        qDebug() << "ExamWorker do not setWorkAndMode!";
        return;
    }

    double scorePart = 0;
    //1.safety level score
    scorePart += examiningSafetyLevel();

    //2.anti virus score
    //scorePart += examiningAntivirus();

    //3.network protection score
    scorePart += examiningNetProtect();

/////////test add score//////////////////////////////
    scorePart += m_mode->getTestScore();


    Q_EMIT sendExamineProcess("finished", 0); //发送检测结束标志
    int scoreTotal = qRound(scorePart);
    m_work->setSafetyScore(scoreTotal); //检测结束储存分数
}

double ExamWorker::examiningSafetyLevel()
{
    double safetyLevelScore = 100 * SafetyLevelScoreCoe;
    int fixItemId = 0;
    double weightValue;
    //1.safety level score
    int safetyLev = m_mode->getSafetyLevel(); //高-1，中-2，低-3
    switch (safetyLev) {
    case 1:
        weightValue = 1.0;
        break;
    case 2:
        weightValue = 0.6;
        fixItemId = 2;
        break;
    case 3:
        weightValue = 0;
        fixItemId = 1;
        break;
    default:
        weightValue = 0;
    }
    Q_EMIT sendExamineProcess(tr("Checking security level..."), fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examine safety level finished!";
    safetyLevelScore *= weightValue;
    return safetyLevelScore;
}

double ExamWorker::examiningAntivirus()
{
    double safetyLevelScore = 100 * AntivirusScoreCoe;
    int fixItemId = 0;
    bool isbLatestVer = true; //getVirusBankVer()
    double v = 1; //V系数
    if (isbLatestVer)  v = 1;
    else {
        v = 0.5;
        fixItemId = 3;
    }
    Q_EMIT sendExamineProcess("Checking virus bank version...", fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examine virus bank version finished!";

    double m = 1; //M系数
    int latestKillSty = m_mode->getLastKillStyle(); //getLatestKillSty 1-全盘，2-快速，3-指定
    switch (latestKillSty) {
    case 1:
        m = 1.0;
        break;
    case 2:
        m = 0.8;
        fixItemId = 4;
        break;
    case 3:
        m = 0.6;
        fixItemId = 4;
        break;
    default:
        m = 0;
        fixItemId = 4;
        break;
    }
    Q_EMIT sendExamineProcess("Checking latest kill style...", fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examine latest kill style finished!";

    double t = 1; //T系数
    int days = 0;

    QDateTime lastCheckTime = m_mode->getLastKillTime();
    qint64 deltaTime = lastCheckTime.secsTo(QDateTime::currentDateTime());

    float minutes = deltaTime / 60;
    float hours = minutes / 60;
    days = int(hours / 24);
    qDebug() << "last kill time to today: " << days << "days";

    if (15 > days) t = 1;
    else if (60 > days){
        t = (60 - days) / 45;
        fixItemId = 5;
    } else t = 0;
    Q_EMIT sendExamineProcess("Checking latest killed days...", fixItemId);
    waitWithStatus(1000);
    qDebug() << "Examining -- examine latest killed days finished!";

    safetyLevelScore *= v * m * t;
    Q_EMIT sendExamineProcess("Checking anti virus...", 0);
    waitWithStatus(1000);
    qDebug() << "Examining -- examine anti virus finished!";

    return  safetyLevelScore;
}

double ExamWorker::examiningNetProtect()
{
    //2019.12.6未更新
    return 100.0 * NetProtectScoreCoe;
}

